<?php

namespace AsyncAws\Sqs\Exception;

use AsyncAws\Core\Exception\Http\ClientException;

/**
 * The specified attribute doesn't exist.
 */
final class InvalidAttributeNameException extends ClientException
{
}
