<?php

namespace AsyncAws\Sqs\Exception;

use AsyncAws\Core\Exception\Http\ClientException;

/**
 * Error code 400. Unsupported operation.
 */
final class UnsupportedOperationException extends ClientException
{
}
