<?php

namespace AsyncAws\Sqs\Exception;

use AsyncAws\Core\Exception\Http\ClientException;

/**
 * A queue attribute value is invalid.
 */
final class InvalidAttributeValueException extends ClientException
{
}
